import { _decorator, Camera, Component, IVec2Like, Mat4, Node, UITransform, Vec3 } from 'cc';
const { ccclass, property } = _decorator;

const __vec3_temp1 = new Vec3;

@ccclass('GuideInRect')
export class GuideInRect extends Component {
    @property(Camera)
    readonly camera: Camera;
    @property(UITransform)
    readonly rectTransform: UITransform;
    @property(UITransform)
    readonly guideTransform: UITransform;
    @property(Node)
    readonly guideDirection: Node;
    @property(Node)
    sourceNode: Node;
    @property(Node)
    targetNode: Node;

    protected onLoad(): void {
        console.assert(this.guideTransform.node.parent == this.rectTransform.node);
    }

    lateUpdate(deltaTime: number) {
        this.updateGuidePosition();
    }

    protected updateGuidePosition(): void {
        const camera = this.camera;
        const targetPosition = camera.convertToUINode(this.targetNode.worldPosition, this.rectTransform.node, __vec3_temp1);
        const rectHalfWidth = this.rectTransform.width * this.rectTransform.node.scale.x * 0.5;
        const rectHalfHeight = this.rectTransform.height * this.rectTransform.node.scale.y * 0.5;
        const position = this.guideTransform.node.position = GuideInRect.closestPointOnRectangleEdges(targetPosition.x, targetPosition.y, rectHalfWidth, rectHalfHeight, targetPosition);
        this.guideDirection.angle = Math.atan2(position.y, position.x) / Math.PI * 180;
    }

    public static closestPointOnRectangleEdges<T extends IVec2Like>(x: number, y: number, halfWidth: number, halfHeight: number, out: T): T {
        let closestX: number, closestY: number;
        let xInside: boolean, yInside: boolean;
        if (x <= -halfWidth) {
            closestX = -halfWidth;
        } else if (x >= halfWidth) {
            closestX = halfWidth;
        } else {
            closestX = x;
            xInside = true;
        }

        if (y <= -halfHeight) {
            closestY = -halfHeight;
        } else if (y >= halfHeight) {
            closestY = halfHeight;
        } else {
            closestY = y;
            yInside = true;
        }

        if (xInside && yInside) {
            closestX = x;
            closestY = y;
        }

        out.x = closestX, out.y = closestY;
        return out;
    }
}